@@ -1723,7 +1723,7 @@ __imlib_GetBlendFunction(ImlibOp op, char blend, char merge_alpha, char rgb_src,
       return NULL;
 
 #ifdef DO_MMX_ASM
-   do_mmx = ! !(__imlib_get_cpuid() & CPUID_MMX);
+   do_mmx = !!(__imlib_get_cpuid() & CPUID_MMX);
 #elif DO_AMD64_ASM
    do_mmx = 1;                  // instruction set is always present
 #endif
@@ -1731,7 +1731,7 @@ __imlib_GetBlendFunction(ImlibOp op, char blend, char merge_alpha, char rgb_src,
       blend = 0;
    if (blend && cm && rgb_src && (A_CMOD(cm, 0xff) == 0))
       return NULL;
-   return ibfuncs[! !do_mmx][opi][! !cm][! !merge_alpha][! !rgb_src][! !blend];
+   return ibfuncs[!!do_mmx][opi][!!cm][!!merge_alpha][!!rgb_src][!!blend];
 }
 
 void
@@ -1894,10 +1894,6 @@ __imlib_BlendImageToImage(ImlibImage * im_src, ImlibImage * im_dst,
              if ((dw < 1) || (dh < 1))
                 return;
           }
-        if (psx != dx)
-           sx += ((dx - psx) * ssw) / abs(ddw);
-        if (psy != dy)
-           sy += ((dy - psy) * ssh) / abs(ddh);
         if (psw != dw)
            sw = (sw * dw) / psw;
         if (psh != dh)
