@@ -1,7 +1,4 @@
 #include "loader_common.h"
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 #include <gif_lib.h>
 
 int
@@ -57,7 +54,7 @@ load2(ImlibImage * im, int load_data)
              if (!IMAGE_DIMENSIONS_OK(w, h))
                 goto quit;
 
-             rows = calloc(h, sizeof(GifRowType *));
+             rows = calloc(h, sizeof(GifRowType));
              if (!rows)
                 goto quit;
 
