@@ -1,6 +1,4 @@
 #include "loader_common.h"
-
-#include <fcntl.h>
 #include <sys/stat.h>
 #include <webp/decode.h>
 #include <webp/encode.h>
@@ -37,16 +35,16 @@ load2(ImlibImage * im, int load_data)
    int                 rc;
    uint8_t            *encoded_data;
    struct stat         stats;
-   int                 encoded_fd;
+   int                 fd;
    WebPBitstreamFeatures features;
    VP8StatusCode       vp8return;
    unsigned int        size;
 
-   encoded_fd = fileno(im->fp);
-   if (encoded_fd < 0)
+   fd = fileno(im->fp);
+   if (fd < 0)
       return LOAD_FAIL;
 
-   if (fstat(encoded_fd, &stats) < 0)
+   if (fstat(fd, &stats) < 0)
       return LOAD_FAIL;
 
    rc = LOAD_FAIL;
@@ -57,7 +55,7 @@ load2(ImlibImage * im, int load_data)
 
    /* Check signature */
    size = 12;
-   if (read(encoded_fd, encoded_data, size) != (long)size)
+   if (read(fd, encoded_data, size) != (long)size)
       goto quit;
    if (memcmp(encoded_data + 0, "RIFF", 4) != 0 ||
        memcmp(encoded_data + 8, "WEBP", 4) != 0)
@@ -68,7 +66,7 @@ load2(ImlibImage * im, int load_data)
       goto quit;
 
    size -= 12;
-   if (read(encoded_fd, encoded_data + 12, size) != (long)size)
+   if (read(fd, encoded_data + 12, size) != (long)size)
       goto quit;
 
    if (WebPGetInfo(encoded_data, stats.st_size, &im->w, &im->h) == 0)
@@ -121,16 +119,15 @@ load2(ImlibImage * im, int load_data)
 char
 save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
 {
+   FILE               *f;
    int                 rc;
-   int                 encoded_fd;
    ImlibImageTag      *quality_tag;
    float               quality;
    uint8_t            *encoded_data;
-   ssize_t             encoded_size;
+   size_t              encoded_size;
 
-   encoded_fd = open(im->real_file,
-                     O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
-   if (encoded_fd < 0)
+   f = fopen(im->real_file, "wb");
+   if (!f)
       return LOAD_FAIL;
 
    rc = LOAD_FAIL;
@@ -161,7 +158,7 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    encoded_size = WebPEncodeBGRA((uint8_t *) im->data, im->w, im->h,
                                  im->w * 4, quality, &encoded_data);
 
-   if (write(encoded_fd, encoded_data, encoded_size) < encoded_size)
+   if (fwrite(encoded_data, encoded_size, 1, f) != encoded_size)
       goto quit;
 
    rc = LOAD_SUCCESS;
@@ -169,7 +166,7 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
  quit:
    if (encoded_data)
       WebPFree(encoded_data);
-   close(encoded_fd);
+   fclose(f);
 
    return rc;
 }
